#include <stdlib.h>
#include <string.h>
#include <fftw3.h>

#include "fft.h"

fft_info * alloc_fft(int sample_rate)
{
	fft_info *pnt = (fft_info *)malloc(sizeof(fft_info));
	if (!pnt)
		return NULL;

	memset(pnt, 0x00, sizeof(fft_info));

	pnt -> pout = fftw_malloc(sizeof(fftw_complex) * sample_rate + 1);
	pnt -> pin  = (double *)malloc(sizeof(double) * sample_rate);

	/* init fftw */
	pnt -> plan = fftw_plan_dft_r2c_1d(sample_rate, pnt -> pin, pnt -> pout, FFTW_MEASURE);

	return pnt;
}

void free_fft(fft_info *pnt)
{
	free(pnt -> pin);
	fftw_free(pnt -> pout);

	free(pnt);
}

/* this function does a fft on the data pointed to by 'in'
 * if in points to a multi-channel sample, one must select the channel to do the fft on: only
 * one channel per invocation is fftd
 * it expects 'sample_rate' samples in in for each channel
 * out must be sample_rate/2 in size (round it up if sample_rate is odd!)
 */
void do_fft(fft_info *finfo, double *in, int sample_rate, double *out)
{
	int loop;

	for(loop=0; loop<sample_rate; loop++)
	{
		(finfo -> pin)[loop] = in[loop];
	}

	/* calc fft */
	fftw_execute(finfo -> plan);

	for(loop=0; loop<((sample_rate / 2) + 1); loop++)
	{
		out[loop] = (finfo -> pout)[0][loop];
	}
}
